@@ -14,7 +14,7 @@ Math::Symbolic::Custom::Polynomial - Polynomial routines for Math::Symbolic
 
 =head1 VERSION
 
-Version 0.2
+Version 0.3
 
 =cut
 
@@ -23,7 +23,7 @@ require Exporter;
 our @ISA = qw(Exporter);
 our @EXPORT = qw/symbolic_poly/;
 
-our $VERSION = '0.2';
+our $VERSION = '0.3';
 
 use Math::Symbolic qw(:all);
 use Math::Symbolic::Custom::Collect 0.32;
@@ -33,7 +33,7 @@ BEGIN {
     *import = \&Math::Symbolic::Custom::Base::aggregate_import
 }
    
-our $Aggregate_Export = [qw/test_polynomial/];
+our $Aggregate_Export = [qw/test_polynomial apply_synthetic_division apply_polynomial_division/];
 
 # attempt to circumvent above redefinition of import function
 Math::Symbolic::Custom::Polynomial->export_to_level(1, undef, 'symbolic_poly');
@@ -46,7 +46,9 @@ This is the beginnings of a module to provide some polynomial utility routines f
 
 "symbolic_poly()" creates a polynomial Math::Symbolic expression according to the supplied variable and 
 coefficients, and "test_polynomial()" attempts the inverse, it looks at a Math::Symbolic expression and 
-tries to extract polynomial coefficients (so long as the expression represents a polynomial).
+tries to extract polynomial coefficients (so long as the expression represents a polynomial). The 
+"apply_synthetic_division()" and "apply_polynomial_division()" methods will attempt to perform division 
+on a target polynomial. 
 
 =head1 EXAMPLE
 
@@ -108,43 +110,16 @@ tries to extract polynomial coefficients (so long as the expression represents a
     print "Putting back into original polynomial:-\n\tat k = 3 and x = $root1:\t", 
             $some_poly->value('k' => 3, 'x' => $root1), 
             "\n\tat k = 3 and x = $root2:\t", 
-            $some_poly->value('k' => 3, 'x' => $root2), "\n\n";
-            
-    # finding roots with Math::Polynomial::Solve
-    use Math::Polynomial::Solve qw(poly_roots coefficients);
-    coefficients order => 'descending';
+            $some_poly->value('k' => 3, 'x' => $root2), "\n\n";          
 
-    # some big polynomial
-    my $big_poly = parse_from_string("phi^8 + 3*phi^7 - 5*phi^6 + 2*phi^5 -7*phi^4 + phi^3 + phi^2 - 2*phi + 9");
-    # if test_polynomial() is not supplied with the indeterminate variable, it will try to autodetect
-    my ($var, $co) = $big_poly->test_polynomial();  
-    my @coeffs = @{$co};
-    my $degree = scalar(@coeffs)-1;
-
-    print "'$big_poly' is a polynomial in $var of degree $degree with " . 
-                "coefficients (ordered in descending powers): (", join(", ", @coeffs), ")\n";
-
-    # Find the roots of the polynomial using Math::Polynomial::Solve. 
-    my @roots = poly_roots( 
-          # call value() on each coefficient to get a number.
-          # if there were any parameters, we would have to supply their value
-          # here to force the coefficients down to a number.
-          map { $_->value() } @coeffs 
-          );
-
-    print "The roots and corresponding values of the polynomial are:-\n";
-    foreach my $root (@roots) {
-          # put back into the original expression to verify
-          my $val = $big_poly->value('phi' => $root);
-          print "\t$root => $val\n";
-    }
-
-=head1 symbolic_poly
+=head1 symbolic_poly()
 
 Exported by default (or it should be; try calling it directly if that fails). Constructs a Math::Symbolic 
 expression corresponding to the passed parameters: a symbol for the desired indeterminate variable, and an array 
 ref to the coefficients in descending order (which can also be symbols).
 
+See the 'Example' section above for examples of use.
+
 =cut
 
 sub symbolic_poly {
@@ -154,7 +129,13 @@ sub symbolic_poly {
     return undef unless defined($coeffs) and (ref($coeffs) eq 'ARRAY');
 
     my @c = reverse @{ $coeffs };       # reverse the coefficient array as it will be built in ascending order
-    return undef unless scalar(@c) > 1;
+    return undef unless scalar(@c) > 0;     # TODO: perhaps should return 0?
+
+    if ( scalar(@c) == 1 ) {
+        my $single = $c[0];
+        $single = parse_from_string($single) unless ref($single) =~ /^Math::Symbolic/;
+        return $single;
+    }
 
     $var = parse_from_string($var) unless ref($var) =~ /^Math::Symbolic/;
     return undef unless defined $var;
@@ -206,17 +187,53 @@ sub symbolic_poly {
     return $nt;
 }
 
-=head1 test_polynomial
+=head1 method test_polynomial()
 
 Exported through the Math::Symbolic module extension class. Call it on a polynomial Math::Symbolic expression and it will 
 try to determine the coefficient expressions. 
 
 Takes one parameter, the indeterminate variable. If this is not provided, test_polynomial will try to detect the variable. This
-can be useful to test if a Math::Symbolic expression looks like a polynomial.
+can be useful to test if an arbitrary Math::Symbolic expression looks like a polynomial.
  
 If the expression looks like a polynomial of degree 2, then it will apply the quadratic equation to produce
 expressions for the roots, and the discriminant.
 
+Example (also see 'Example' section above):
+
+    use strict;
+    use Math::Symbolic qw(:all);
+    use Math::Symbolic::Custom::Polynomial;
+    use Math::Complex;
+
+    # finding roots with Math::Polynomial::Solve
+    use Math::Polynomial::Solve qw(poly_roots coefficients);
+    coefficients order => 'descending';
+
+    # some big polynomial
+    my $big_poly = parse_from_string("phi^8 + 3*phi^7 - 5*phi^6 + 2*phi^5 -7*phi^4 + phi^3 + phi^2 - 2*phi + 9");
+    # if test_polynomial() is not supplied with the indeterminate variable, it will try to autodetect
+    my ($var, $co) = $big_poly->test_polynomial();  
+    my @coeffs = @{$co};
+    my $degree = scalar(@coeffs)-1;
+
+    print "'$big_poly' is a polynomial in $var of degree $degree with " . 
+                "coefficients (ordered in descending powers): (", join(", ", @coeffs), ")\n";
+
+    # Find the roots of the polynomial using Math::Polynomial::Solve. 
+    my @roots = poly_roots( 
+          # call value() on each coefficient to get a number.
+          # if there were any parameters, we would have to supply their value
+          # here to force the coefficients down to a number.
+          map { $_->value() } @coeffs 
+          );
+
+    print "The roots and corresponding values of the polynomial are:-\n";
+    foreach my $root (@roots) {
+          # put back into the original expression to verify
+          my $val = $big_poly->value('phi' => $root);
+          print "\t$root => $val\n";
+    }
+
 =cut
 
 sub test_polynomial {
@@ -280,7 +297,6 @@ sub test_polynomial {
     push @constants, Math::Symbolic::Constant->new($terms->{constant_accumulator});
     delete $terms->{constant_accumulator};
 
-    # ...assume we've figured out the indeterminate of the polynomial
     my @vars = grep { $trees->{$_}{name} eq $var } grep { /^VAR/ } keys %{$trees};
     if ( scalar(@vars) == 0 ) {
         carp("Passed indeterminate '$ind' but no variable in expression matches.");
@@ -403,6 +419,204 @@ sub test_polynomial {
     return wantarray ? ($var, \@coeffs, $discriminant, \@roots) : [$var, \@coeffs, $discriminant, \@roots];
 }
 
+=head1 method apply_synthetic_division()
+
+Exported through the Math::Symbolic module extension class. Divides the target polynomial by a divisor of the 
+form (x - r) using synthetic division. This method relies on test_polynomial() to detect if the target 
+Math::Symbolic expression is a polynomial, and to determine the coefficients and the indeterminate variable if 
+not provided. 
+
+Takes two parameters, the evaluator i.e. the 'r' part of (x-r) (required, can be a Math::Symbolic expression or a 
+text string which will be converted to a Math::Symbolic expression using the parser), and the polynomial variable 
+(optional, as test_polynomial() can try to detect it). 
+
+If called in a scalar context, it returns the polynomial in its divided form, i.e. D*Q + R (where D is the divisor
+x-r, Q is the quotient (polynomial) and R is the remainder).
+
+If called in a list context, it returns the polynomial in divided form as describe above, and also the divisor, 
+quotient and remainder expressions. 
+
+Example:
+
+    use strict;
+    use warnings;
+    use Math::Symbolic qw(:all);
+    use Math::Symbolic::Custom::Polynomial;
+
+    # Divide (2*x^3 - 6*x^2 + 2*x - 1) by (x - 3)
+    my $poly = parse_from_string("2*x^3 - 6*x^2 + 2*x - 1");
+    my $evaluator = 3; # it will put "x - $evaluator" internally.
+
+    # Specifying 'x' as the polynomial variable in apply_synthetic_division() is optional, see test_polynomial().
+    # It is not needed for this straightforward polynomial but is just present for documentation.
+    my ($full_expr, $divisor, $quotient, $remainder) = $poly->apply_synthetic_division($evaluator, 'x');  
+
+    # The return values are Math::Symbolic expressions
+    print "Full expression: $full_expr\n";  # Full expression: ((x - 3) * (2 + (2 * (x ^ 2)))) + 5
+    print "Divisor: $divisor\n";    # Divisor: x - 3
+    print "Quotient: $quotient\n";  # Quotient: 2 + (2 * (x ^ 2))
+    print "Remainder: $remainder\n";    # Remainder: 5
+
+    # Also works symbolically. Divide it by r.
+    ($full_expr, $divisor, $quotient, $remainder) = $poly->apply_synthetic_division('r');  
+
+    print "Full expression: $full_expr\n";
+    # Full expression: ((x - r) * (((((2 + (2 * (r ^ 2))) + (2 * (x ^ 2))) + ((2 * r) * x)) - (6 * r)) - (6 * x))) + ((((2 * r) + (2 * (r ^ 3))) - 1) - (6 * (r ^ 2)))
+    print "Divisor: $divisor\n";    # Divisor: x - r
+    print "Quotient: $quotient\n";  # Quotient: ((((2 + (2 * (r ^ 2))) + (2 * (x ^ 2))) + ((2 * r) * x)) - (6 * r)) - (6 * x)
+    print "Remainder: $remainder\n";    # Remainder: (((2 * r) + (2 * (r ^ 3))) - 1) - (6 * (r ^ 2))
+
+=cut
+
+sub apply_synthetic_division {
+    my ($f, $r, $ind) = @_;
+
+    return undef unless defined $r;
+
+    $r = parse_from_string($r) unless ref($r) =~ /^Math::Symbolic/;
+    return undef unless defined $r;
+
+    my ($var, $co) = $f->test_polynomial($ind);
+
+    return undef unless defined $var;
+
+    my @coeffs = @{$co};
+    my $degree = scalar(@coeffs)-1;
+
+    return undef if $degree < 2;
+
+    my @quotient;  # Coefficients of Q
+    my $remainder; # Remainder R
+    
+    # Initialize with the leading coefficient
+    my $carry = shift @coeffs;
+    push @quotient, $carry;
+
+    # Perform synthetic division
+    foreach my $coeff (@coeffs) {
+        $carry = Math::Symbolic::Operator->new('+', $coeff, Math::Symbolic::Operator->new('*', $carry, $r));
+        push @quotient, $carry;
+    }
+
+    # The last carry is the remainder
+    $remainder = pop @quotient;
+    $remainder = $remainder->to_collected();
+
+    my $q_poly = symbolic_poly($var, \@quotient);
+    $q_poly = $q_poly->to_collected();
+
+    my $divisor = Math::Symbolic::Operator->new('-', Math::Symbolic::Variable->new($var), $r);
+    my $full_expr = Math::Symbolic::Operator->new('+', Math::Symbolic::Operator->new('*',  $divisor, $q_poly), $remainder);
+
+    return wantarray ? ($full_expr, $divisor, $q_poly, $remainder) : $full_expr;
+}
+
+=head1 method apply_polynomial_division()
+
+Exported through the Math::Symbolic module extension class. Divides the target polynomial by a divisor polynomial
+using polynomial long division. This method relies on test_polynomial() to detect if the target and divisor 
+expressions are polynomials, and to determine the coefficients and the indeterminate variables if not provided. 
+The indeterminate variables must be the same in both expressions.
+
+Takes two parameters, the divisor polynomial (required, can be a Math::Symbolic expression or a text string which
+will be converted to a Math::Symbolic expression using the parser), and the polynomial variable (optional, as 
+test_polynomial() can try to detect it). 
+
+If called in a scalar context, it returns the polynomial in its divided form, i.e. D*Q + R (where D is the divisor, 
+Q is the quotient and R is the remainder).
+
+If called in a list context, it returns the polynomial in divided form as describe above, and also the divisor, 
+quotient and remainder expressions. 
+
+Example:
+
+    use strict;
+    use warnings;
+    use Math::Symbolic qw(:all);
+    use Math::Symbolic::Custom::Polynomial;
+
+    # Divide (2*y^3 - 3*y^2 - 3*y +2) by (y - 2)
+    my $poly = symbolic_poly('y', [2, -3, -3, 2]);
+    my ($full_expr, $divisor, $quotient, $remainder) = $poly->apply_polynomial_division('y-2', 'y');
+
+    print "Full expression: $full_expr\n";  # Full expression: (y - 2) * ((y + (2 * (y ^ 2))) - 1)
+    print "Divisor: $divisor\n";    # Divisor: y - 2
+    print "Quotient: $quotient\n";  # Quotient: (y + (2 * (y ^ 2))) - 1
+    print "Remainder: $remainder\n";    # Remainder: 0
+
+    # Also works symbolically. Divide by (y^2 - 2*k*y + k)
+    ($full_expr, $divisor, $quotient, $remainder) = $poly->apply_polynomial_division('y^2 - 2*k*y + k', 'y');
+
+    print "Full expression: $full_expr\n";
+    # Full expression: ((((y ^ 2) - ((2 * k) * y)) + k) * (((4 * k) + (2 * y)) - 3)) + (((((2 + (3 * k)) + ((8 * (k ^ 2)) * y)) - (4 * (k ^ 2))) - (3 * y)) - ((8 * k) * y))
+    print "Divisor: $divisor\n";    # Divisor: ((y ^ 2) - ((2 * k) * y)) + k
+    print "Quotient: $quotient\n";  # Quotient: ((4 * k) + (2 * y)) - 3
+    print "Remainder: $remainder\n";    # Remainder: ((((2 + (3 * k)) + ((8 * (k ^ 2)) * y)) - (4 * (k ^ 2))) - (3 * y)) - ((8 * k) * y)
+
+=cut
+
+sub apply_polynomial_division {
+    my ($f, $divisor, $ind) = @_;
+
+    return undef unless defined $divisor;
+
+    $divisor = parse_from_string($divisor) unless ref($divisor) =~ /^Math::Symbolic/;
+    return undef unless defined $divisor;
+
+    my ($var1, $co1) = $f->test_polynomial($ind);
+    return undef unless defined $var1;    
+
+    my ($var2, $co2) = $divisor->test_polynomial($ind);
+    return undef unless defined $var2;
+
+    return undef unless $var1 eq $var2;
+
+    my @dividend = @{$co1};  # Coefficients of the dividend polynomial
+    my @divisor = @{$co2};    # Coefficients of the divisor polynomial
+    
+    my $dividend_degree = scalar(@dividend) - 1;
+    my $divisor_degree  = scalar(@divisor) - 1;
+    
+    my @quotient = ();
+    my @remainder = @dividend;      
+
+    # Perform division until the degree of remainder is less than the divisor's degree
+    while (scalar(@remainder) - 1 >= $divisor_degree) {
+
+        my $lead_coeff_ratio = Math::Symbolic::Operator->new('/', $remainder[0], $divisor[0]);
+        push @quotient, $lead_coeff_ratio;
+
+        my $degree_diff = scalar(@remainder) - scalar(@divisor);
+
+        # Subtract the product of the divisor and the term from the remainder
+        my @scaled_divisor = map { Math::Symbolic::Operator->new('*', $_, $lead_coeff_ratio) } (@divisor, (0) x $degree_diff);
+        @remainder = map { Math::Symbolic::Operator->new('-', $remainder[$_], $scaled_divisor[$_]) } (0..scalar(@remainder)-1);
+        
+        shift @remainder;
+    }
+
+    # Collect up the new coefficients
+    @quotient = map { scalar($_->to_collected()) } @quotient;
+    @remainder = map { scalar($_->to_collected()) } @remainder;
+
+    my $q_poly = symbolic_poly($var1, \@quotient);
+    return undef unless defined $q_poly;
+    $q_poly = $q_poly->to_collected();
+
+    my $r_poly = symbolic_poly($var1, \@remainder);
+
+    my $full_expr;
+    if ( defined($r_poly) && !$r_poly->is_identical(Math::Symbolic::Constant->new(0)) ) {
+        $r_poly = $r_poly->to_collected();
+        $full_expr = Math::Symbolic::Operator->new('+', Math::Symbolic::Operator->new('*',  $divisor, $q_poly), $r_poly);
+    }
+    else {
+        $full_expr = Math::Symbolic::Operator->new('*',  $divisor, $q_poly);
+        $r_poly = Math::Symbolic::Constant->new(0) unless defined $r_poly;
+    }
+
+    return wantarray ? ($full_expr, $divisor, $q_poly, $r_poly) : $full_expr;
+}
 
 =head1 SEE ALSO
 
